
#include <iostream>

#include <FL/Fl.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Flex.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Gl_Window.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Menu_Button.H>
#include <FL/Fl_Slider.H>
#include <FL/Fl_Tile.H>
#include <FL/Fl_Value_Input.H>

#define Fl_DockGroup Fl_Group
#define Fl_ResizableBar Fl_Group
#define Fl_StatusBar Fl_Group
#define Fl_TimelineGroup Fl_Group
#define Fl_TimelineWidget Fl_Gl_Window
#define Fl_Viewport Fl_Gl_Window

const char*  gettext(const char* t)
{
    return t;
}


Fl_Box* uiTimelineLimit;
Fl_Double_Window* uiMain;
Fl_DockGroup* uiDock;
Fl_Flex* uiRegion, *uiViewGroup;
Fl_Group* uiMenuGroup, *uiToolsGroup, *uiDockGroup;
Fl_Group* uiTopBar, *uiBottomBar, *uiPixelBar, *uiStatusGroup;
Fl_Menu_Bar* uiMenu_Bar;
Fl_Button* uiEdit, *uiScrub, *uiSelection, *uiDraw, *uiErase, *uiCircle, *uiRectangle, *uiArrow, *uiText, *uiPenColor, *uiOldPenColor, *uiFlipPenColor, *uiUndoDraw, *uiRedoDraw, *uiExposureLess, *uiFStop, *uiExposureMore, *uiGammaSwitch;
Fl_Menu_Button* uiColorChannel, *uiICS, *OCIOLook, *OCIOView;
Fl_ResizableBar* uiResizableBar;
Fl_Slider* uiPenOpacity, *uiGain, *uiGamma;
Fl_TimelineGroup* uiTimelineGroup;
Fl_Tile* uiTileGroup;
Fl_TimelineWidget* uiTimeline;
Fl_Viewport* uiView;
Fl_Value_Input* uiGainInput, *uiGammaInput; 

struct RenderSize
{
    int w = 0;
    int h = 0;
};

static void set_edit_mode_cb()
{
}

static void cb_Resize(Fl_Widget* o, void* d)
{
    std::cerr << "resize called" << std::endl;
    RenderSize renderSize;
    renderSize.w = 1280;
    renderSize.h = 688;
    Fl_Double_Window* mw = uiMain;
    int screen = mw->screen_num();
    float scale = Fl::screen_scale(screen);

    int W = renderSize.w;
    int H = renderSize.h;

    int minx, miny, maxW, maxH, posX, posY;
    Fl::screen_work_area(minx, miny, maxW, maxH, screen);

    posX = mw->x();
    posY = mw->y();

    int decW = mw->decorated_w();
    int decH = mw->decorated_h();

    int dW = decW - mw->w();
    int dH = decH - mw->h();

    maxW -= dW;
    maxH -= dH;
    posX += dW / 2;
#ifdef _WIN32
    miny += dH - dW / 2;
#endif

    // Take into account the different UI bars
    if (uiMenuGroup->visible())
        H += uiMenuGroup->h();

    if (uiTopBar->visible())
        H += uiTopBar->h();

    if (uiPixelBar->visible())
        H += uiPixelBar->h();

    if (uiBottomBar->visible())
    {
        H += uiBottomBar->h();
    }

    if (uiStatusGroup->visible())
        H += uiStatusGroup->h();

    if (uiToolsGroup->visible())
        W += uiToolsGroup->w();

    if (uiDockGroup->visible())
        W += uiDockGroup->w();

    maxW = (int)(maxW / scale);
    if (W < 690)
    {
        W = 690;
    }
    else if (W > maxW)
    {
        W = maxW;
    }

    maxH = (int)(maxH / scale);
    if (H < 602)
    {
        H = 602;
    }
    else if (H > maxH)
    {
        H = maxH;
    }

    if (uiBottomBar->visible())
    {
        H += uiBottomBar->h();
    }

    //
    // Final sanity checks.
    //
    if (W > maxW)
        W = maxW;
    if (H > maxH)
        H = maxH;

    if (posX + W > maxW)
        posX = minx;
    if (posY + H > maxH)
        posY = miny;

    std::cerr << "mw resize=" << posX << ", " << posY
              << " " << W << "x" << H << std::endl;
    mw->resize(posX, posY, W, H);

    std::cerr << "1 uiView->y()=" << uiView->y() << std::endl;
    uiRegion->layout();
    std::cerr << "2 uiView->y()=" << uiView->y() << std::endl;

    // Set_mode_cb();

}

static void cb_Timeout(void* d)
{
    std::cerr << "timeout called" << std::endl;
    cb_Resize((Fl_Widget*)d,d);
}


int main(const int argc, const char* argv[])
{
    uiMain = new Fl_Double_Window(690, 602);
    { Fl_Flex* o = uiRegion = new Fl_Flex(0, 0, 690, 602);
        uiRegion->box(FL_FLAT_BOX);
        uiRegion->color(FL_BACKGROUND_COLOR);
        uiRegion->selection_color(FL_BACKGROUND_COLOR);
        uiRegion->labeltype(FL_NORMAL_LABEL);
        uiRegion->labelfont(0);
        uiRegion->labelsize(14);
        uiRegion->labelcolor(FL_FOREGROUND_COLOR);
        uiRegion->align(Fl_Align(FL_ALIGN_TOP));
        uiRegion->when(FL_WHEN_RELEASE);
        { uiMenuGroup = new Fl_Group(0, 0, 690, 25);
            uiMenuGroup->box(FL_FLAT_BOX);
            { Fl_Group* o = new Fl_Group(0, 0, 660, 25);
                { uiMenu_Bar = new Fl_Menu_Bar(0, 0, 602, 25);
                    uiMenu_Bar->box(FL_UP_BOX);
                    uiMenu_Bar->color(FL_BACKGROUND_COLOR);
                    uiMenu_Bar->selection_color(FL_SELECTION_COLOR);
                    uiMenu_Bar->labeltype(FL_NO_LABEL);
                    uiMenu_Bar->labelfont(0);
                    uiMenu_Bar->labelsize(14);
                    uiMenu_Bar->labelcolor(FL_FOREGROUND_COLOR);
                    uiMenu_Bar->textsize(12);
                    uiMenu_Bar->align(Fl_Align(FL_ALIGN_CENTER));
                    uiMenu_Bar->when(FL_WHEN_RELEASE_ALWAYS);
                } // Fl_Menu_Bar* uiMenu_Bar
                { uiEdit = new Fl_Button(602, 0, 58, 25, gettext("Edit"));
//tooltip(gettext("Toggle Edit Mode"));
                    uiEdit->down_box(FL_DOWN_BOX);
                    uiEdit->selection_color(FL_YELLOW);
                    uiEdit->labelsize(12);
                } // Fl_Button* uiEdit
                o->end();
            } // Fl_Group* o
            { Fl_Button* o = new Fl_Button(660, 0, 30, 25, "CLICK ME");
                o->tooltip(gettext("Click me to show the bug"));
                o->box(FL_UP_BOX);
                o->color(FL_YELLOW);
                o->selection_color(FL_BACKGROUND_COLOR);
//image( image_WindowFullScreen() );
                o->labeltype(FL_NORMAL_LABEL);
                o->labelfont(0);
                o->labelsize(14);
                o->labelcolor(FL_FOREGROUND_COLOR);
                o->callback((Fl_Callback*)cb_Resize, (void*)(0));
                o->align(Fl_Align(FL_ALIGN_CENTER));
                o->when(FL_WHEN_RELEASE);
            } // Fl_Button* o
            uiMenuGroup->end();
        } // Fl_Group* uiMenuGroup
        { uiTopBar = new Fl_Group(0, 25, 690, 28);
            uiTopBar->box(FL_ENGRAVED_BOX);
            { Fl_Group* o = new Fl_Group(0, 25, 277, 27, gettext("Layers, Exposure and Gain"));
                o->labeltype(FL_NO_LABEL);
                { Fl_Group* o = new Fl_Group(0, 26, 129, 26, gettext("Layers"));
                    o->color((Fl_Color)1397969664);
                    o->labeltype(FL_NO_LABEL);
                    { uiColorChannel = new Fl_Menu_Button(0, 26, 129, 25, gettext("(no image)"));
//tooltip(gettext("Allows you to select different image channels or layers."));
                        uiColorChannel->box(FL_UP_BOX);
                        uiColorChannel->down_box(FL_DOWN_BOX);
                        uiColorChannel->color(FL_BACKGROUND_COLOR);
                        uiColorChannel->selection_color(FL_SELECTION_COLOR);
                        uiColorChannel->labeltype(FL_NORMAL_LABEL);
                        uiColorChannel->labelfont(0);
                        uiColorChannel->labelsize(10);
                        uiColorChannel->labelcolor(FL_FOREGROUND_COLOR);
                        uiColorChannel->align(Fl_Align(FL_ALIGN_CENTER));
                        uiColorChannel->when(FL_WHEN_RELEASE_ALWAYS);
                        Fl_Group::current()->resizable(uiColorChannel);
                    } // Fl_Menu_Button* uiColorChannel
                    o->end();
                } // Fl_Group* o
                { Fl_Group* o = new Fl_Group(129, 25, 148, 26, gettext("Exposure and Gain"));
                    o->labeltype(FL_NO_LABEL);
                    { Fl_Group* o = new Fl_Group(129, 25, 120, 26, gettext("Resizable Control Group"));
                        o->labeltype(FL_NO_LABEL);
                        o->hide();
                        o->end();
                        Fl_Group::current()->resizable(o);
                    } // Fl_Group* o
                    { Fl_Group* o = new Fl_Group(129, 25, 49, 26, gettext("Exposure"));
                        o->labeltype(FL_NO_LABEL);
                        { uiExposureLess = new Fl_Button(129, 26, 12, 25, gettext("@<"));
//tooltip(gettext("Reduce gain 1/2 stop (divide by sqrt(2))."));
                            uiExposureLess->labelsize(10);
                        } // Fl_Button* uiExposureLess
                        { uiFStop = new Fl_Button(141, 26, 25, 25, gettext("f/8"));
//tooltip(gettext("Toggle gain between 1 and the previous setting."));
                        } // Fl_Button* uiFStop
                        { uiExposureMore = new Fl_Button(166, 26, 12, 25, gettext("@>"));
//tooltip(gettext("Increase gain 1/2 stop (multiply by sqrt(2))."));
                            uiExposureMore->labelsize(10);
                        } // Fl_Button* uiExposureMore
                        o->end();
                    } // Fl_Group* o
                    { Fl_Group* o = new Fl_Group(178, 25, 99, 26, gettext("Gain"));
                        o->labeltype(FL_NO_LABEL);
                        { Fl_Value_Input* o = uiGainInput = new Fl_Value_Input(178, 25, 37, 26);
//tooltip(gettext("Gain image value.  A simple multiplier of the image pixel values."));
                            uiGainInput->selection_color(FL_FOREGROUND_COLOR);
                            uiGainInput->maximum(64);
                            uiGainInput->step(0.02);
                            uiGainInput->value(1);
                            o->textsize(12);
                            o->textcolor( FL_BLACK );
                        } // Fl_Value_Input* uiGainInput
                        { Fl_Slider* o = uiGain = new Fl_Slider(215, 25, 59, 26);
//tooltip(gettext("Allows you to adjust the gain or exposure of the image."));
                            uiGain->type(5);
                            uiGain->box(FL_DOWN_BOX);
                            uiGain->color(FL_BACKGROUND_COLOR);
                            uiGain->selection_color(FL_BACKGROUND_COLOR);
                            uiGain->labeltype(FL_NORMAL_LABEL);
                            uiGain->labelfont(0);
                            uiGain->labelsize(10);
                            uiGain->labelcolor(FL_FOREGROUND_COLOR);
                            uiGain->maximum(64);
                            uiGain->value(1);
                            uiGain->align(Fl_Align(FL_ALIGN_CENTER));
                            uiGain->when(FL_WHEN_CHANGED);
                            Fl_Group::current()->resizable(uiGain);
                            o->minimum( 0.02 );
                            o->maximum( 64.0 ); o->step( 0.1 );
                            o->value( 1.0 );
                        } // Fl_Slider* uiGain
                        o->end();
                    } // Fl_Group* o
                    o->end();
                } // Fl_Group* o
                o->end();
            } // Fl_Group* o
            { Fl_Group* o = new Fl_Group(277, 25, 413, 28, gettext("ICS, View, Look and Gamma"));
                o->labeltype(FL_NO_LABEL);
                { Fl_Group* o = new Fl_Group(277, 25, 317, 26);
                    { Fl_Group* o = new Fl_Group(360, 25, 234, 26, gettext("Resizable Control Group"));
                        o->labeltype(FL_NO_LABEL);
                        o->hide();
                        o->end();
                        Fl_Group::current()->resizable(o);
                    } // Fl_Group* o
                    { Fl_Menu_Button* o = uiICS = new Fl_Menu_Button(277, 25, 143, 26, gettext("Input Color Space"));
//tooltip(gettext("OCIO input color space for the image"));
                        uiICS->box(FL_UP_BOX);
                        uiICS->down_box(FL_DOWN_BOX);
                        uiICS->color(FL_BACKGROUND_COLOR);
                        uiICS->selection_color(FL_SELECTION_COLOR);
                        uiICS->labeltype(FL_NORMAL_LABEL);
                        uiICS->labelfont(0);
                        uiICS->labelsize(10);
                        uiICS->labelcolor(FL_FOREGROUND_COLOR);
                        uiICS->align(Fl_Align(96|FL_ALIGN_INSIDE));
                        uiICS->when(FL_WHEN_RELEASE_ALWAYS);
//disable_submenus();
                    } // Fl_Menu_Button* uiICS
                    { Fl_Group* o = new Fl_Group(420, 25, 174, 26, gettext("View and Look"));
                        o->labeltype(FL_NO_LABEL);
                        { Fl_Menu_Button* o = OCIOView = new Fl_Menu_Button(420, 25, 144, 26, gettext("OCIO"));
//tooltip(gettext("OCIO Display/View"));
                            OCIOView->box(FL_UP_BOX);
                            OCIOView->down_box(FL_DOWN_BOX);
                            OCIOView->color(FL_BACKGROUND_COLOR);
                            OCIOView->selection_color(FL_SELECTION_COLOR);
                            OCIOView->labeltype(FL_NORMAL_LABEL);
                            OCIOView->labelfont(0);
                            OCIOView->labelsize(10);
                            OCIOView->labelcolor(FL_FOREGROUND_COLOR);
                            OCIOView->align(Fl_Align(FL_ALIGN_CLIP|FL_ALIGN_INSIDE));
                            OCIOView->when(FL_WHEN_RELEASE_ALWAYS);
//disable_submenus();
                        } // Fl_Menu_Button* OCIOView
                        { Fl_Menu_Button* o = OCIOLook = new Fl_Menu_Button(564, 25, 30, 26, gettext("Look"));
//tooltip(gettext("OCIO Looks"));
                            OCIOLook->box(FL_UP_BOX);
                            OCIOLook->down_box(FL_DOWN_BOX);
                            OCIOLook->color(FL_BACKGROUND_COLOR);
                            OCIOLook->selection_color(FL_SELECTION_COLOR);
                            OCIOLook->labeltype(FL_NORMAL_LABEL);
                            OCIOLook->labelfont(0);
                            OCIOLook->labelsize(10);
                            OCIOLook->labelcolor(FL_FOREGROUND_COLOR);
                            OCIOLook->align(Fl_Align(FL_ALIGN_CLIP|FL_ALIGN_INSIDE));
                            OCIOLook->when(FL_WHEN_RELEASE_ALWAYS);
//disable_glyph();
                        } // Fl_Menu_Button* OCIOLook
                        { Fl_Group* o = new Fl_Group(520, 25, 74, 26, gettext("Resizable Control Group"));
                            o->labeltype(FL_NO_LABEL);
                            o->hide();
                            o->end();
                            Fl_Group::current()->resizable(o);
                        } // Fl_Group* o
                        o->end();
                    } // Fl_Group* o
                    o->end();
                } // Fl_Group* o
                { Fl_Group* o = new Fl_Group(594, 25, 96, 26, gettext("Gamma"));
                    o->labeltype(FL_NO_LABEL);
                    { uiGammaSwitch = new Fl_Button(594, 26, 22, 25, gettext("Y"));
//tooltip(gettext("Toggle gamma between 1 and the previous setting."));
                    } // Fl_Button* uiGammaSwitch
                    { Fl_Group* o = new Fl_Group(610, 25, 62, 26, gettext("Resizable Control Group"));
                        o->hide();
                        o->end();
                        Fl_Group::current()->resizable(o);
                    } // Fl_Group* o
                    { Fl_Group* o = new Fl_Group(616, 25, 74, 26, gettext("Gamma Input"));
                        o->labeltype(FL_NO_LABEL);
                        { Fl_Value_Input* o = uiGammaInput = new Fl_Value_Input(616, 25, 35, 26);
//tooltip(gettext("Allows you to adjust gamma curve for display.\nValue is:  pow( 2, 1/x );"));
                            uiGammaInput->selection_color(FL_FOREGROUND_COLOR);
                            uiGammaInput->maximum(64);
                            uiGammaInput->value(1);
                            o->textsize(12);
                            o->textcolor( FL_BLACK );
                            o->color((Fl_Color)0xf98a8a800);
                        } // Fl_Value_Input* uiGammaInput
                        { uiGamma = new Fl_Slider(651, 26, 39, 25);
//tooltip(gettext("Allows you to adjust gamma curve for display.\nValue is:  pow( 2, 1/x );"));
                            uiGamma->type(5);
                            uiGamma->box(FL_DOWN_BOX);
                            uiGamma->color(FL_BACKGROUND_COLOR);
                            uiGamma->selection_color(FL_BACKGROUND_COLOR);
                            uiGamma->labeltype(FL_NORMAL_LABEL);
                            uiGamma->labelfont(0);
                            uiGamma->labelsize(10);
                            uiGamma->labelcolor(FL_FOREGROUND_COLOR);
                            uiGamma->maximum(4);
                            uiGamma->step(0.01);
                            uiGamma->value(1);
                            uiGamma->align(Fl_Align(FL_ALIGN_BOTTOM));
                            uiGamma->when(FL_WHEN_CHANGED);
                        } // Fl_Slider* uiGamma
                        o->end();
                    } // Fl_Group* o
                    o->end();
                } // Fl_Group* o
                { Fl_Group* o = new Fl_Group(277, 25, 370, 26, gettext("Resizable Control Group"));
                    o->hide();
                    o->end();
                    Fl_Group::current()->resizable(o);
                } // Fl_Group* o
                o->end();
            } // Fl_Group* o
            { Fl_Group* o = new Fl_Group(120, 25, 570, 26);
                o->hide();
                o->end();
                Fl_Group::current()->resizable(o);
            } // Fl_Group* o
            uiTopBar->end();
        } // Fl_Group* uiTopBar
        { uiTileGroup = new Fl_Tile(0, 53, 690, 465);
            uiTileGroup->box(FL_NO_BOX);
            uiTileGroup->color(FL_BACKGROUND_COLOR);
            uiTileGroup->selection_color(FL_BACKGROUND_COLOR);
            uiTileGroup->labeltype(FL_NORMAL_LABEL);
            uiTileGroup->labelfont(0);
            uiTileGroup->labelsize(14);
            uiTileGroup->labelcolor(FL_FOREGROUND_COLOR);
            uiTileGroup->align(Fl_Align(FL_ALIGN_TOP));
            uiTileGroup->when(FL_WHEN_RELEASE);
            { uiTimelineLimit = new Fl_Box(0, 80, 690, 408);
                uiTimelineLimit->labeltype(FL_NO_LABEL);
                uiTimelineLimit->hide();
                Fl_Group::current()->resizable(uiTimelineLimit);
            } // Fl_Box* uiTimelineLimit
            { Fl_Flex* o = uiViewGroup = new Fl_Flex(0, 53, 690, 435);
                uiViewGroup->type(1);
                uiViewGroup->box(FL_FLAT_BOX);
                uiViewGroup->color(FL_BACKGROUND_COLOR);
                uiViewGroup->selection_color(FL_BACKGROUND_COLOR);
                uiViewGroup->labeltype(FL_NORMAL_LABEL);
                uiViewGroup->labelfont(0);
                uiViewGroup->labelsize(14);
                uiViewGroup->labelcolor(FL_FOREGROUND_COLOR);
                uiViewGroup->align(Fl_Align(FL_ALIGN_TOP));
                uiViewGroup->when(FL_WHEN_RELEASE);
                { uiToolsGroup = new Fl_Group(0, 53, 42, 435);
                    uiToolsGroup->box(FL_FLAT_BOX);
                    { uiScrub = new Fl_Button(2, 53, 40, 35);
//tooltip(gettext("Scrubbing Tool"));
                        uiScrub->box(FL_UP_BOX);
                        uiScrub->down_box(FL_DOWN_BOX);
                        uiScrub->value(1);
                        uiScrub->color(FL_BACKGROUND_COLOR);
                        uiScrub->selection_color(FL_YELLOW);
//image( image_Scrub() );
                        uiScrub->labeltype(FL_NORMAL_LABEL);
                        uiScrub->labelfont(0);
                        uiScrub->labelsize(14);
                        uiScrub->labelcolor(FL_FOREGROUND_COLOR);
                        uiScrub->align(Fl_Align(FL_ALIGN_CENTER));
                        uiScrub->when(FL_WHEN_RELEASE);
                    } // Fl_Button* uiScrub
                    { uiSelection = new Fl_Button(2, 88, 40, 35);
//tooltip(gettext("Area Select Tool"));
                        uiSelection->box(FL_UP_BOX);
                        uiSelection->down_box(FL_DOWN_BOX);
                        uiSelection->color(FL_BACKGROUND_COLOR);
                        uiSelection->selection_color(FL_YELLOW);
//image( image_Selection() );
                        uiSelection->labeltype(FL_NORMAL_LABEL);
                        uiSelection->labelfont(0);
                        uiSelection->labelsize(14);
                        uiSelection->labelcolor(FL_FOREGROUND_COLOR);
                        uiSelection->align(Fl_Align(FL_ALIGN_CENTER));
                        uiSelection->when(FL_WHEN_RELEASE);
                    } // Fl_Button* uiSelection
                    { uiDraw = new Fl_Button(2, 123, 40, 35);
//tooltip(gettext("Freehand Drawing Tool"));
                        uiDraw->box(FL_UP_BOX);
                        uiDraw->down_box(FL_DOWN_BOX);
                        uiDraw->color(FL_BACKGROUND_COLOR);
                        uiDraw->selection_color(FL_YELLOW);
//image( image_Draw() );
                        uiDraw->labeltype(FL_NORMAL_LABEL);
                        uiDraw->labelfont(0);
                        uiDraw->labelsize(14);
                        uiDraw->labelcolor(FL_FOREGROUND_COLOR);
                        uiDraw->align(Fl_Align(FL_ALIGN_CENTER));
                        uiDraw->when(FL_WHEN_RELEASE);
                    } // Fl_Button* uiDraw
                    { uiErase = new Fl_Button(2, 158, 40, 35);
//tooltip(gettext("Eraser Tool"));
                        uiErase->box(FL_UP_BOX);
                        uiErase->down_box(FL_DOWN_BOX);
                        uiErase->color(FL_BACKGROUND_COLOR);
                        uiErase->selection_color(FL_YELLOW);
//image( image_Erase() );
                        uiErase->labeltype(FL_NORMAL_LABEL);
                        uiErase->labelfont(0);
                        uiErase->labelsize(14);
                        uiErase->labelcolor(FL_FOREGROUND_COLOR);
                        uiErase->align(Fl_Align(FL_ALIGN_CENTER));
                        uiErase->when(FL_WHEN_RELEASE);
                    } // Fl_Button* uiErase
                    { uiCircle = new Fl_Button(2, 193, 40, 35);
//tooltip(gettext("Circle Tool"));
                        uiCircle->box(FL_UP_BOX);
                        uiCircle->down_box(FL_DOWN_BOX);
                        uiCircle->color(FL_BACKGROUND_COLOR);
                        uiCircle->selection_color(FL_YELLOW);
//image( image_Circle() );
                        uiCircle->labeltype(FL_NORMAL_LABEL);
                        uiCircle->labelfont(0);
                        uiCircle->labelsize(14);
                        uiCircle->labelcolor(FL_FOREGROUND_COLOR);
                        uiCircle->align(Fl_Align(FL_ALIGN_CENTER));
                        uiCircle->when(FL_WHEN_RELEASE);
                    } // Fl_Button* uiCircle
                    { uiRectangle = new Fl_Button(2, 228, 40, 35);
//tooltip(gettext("Rectangle Tool"));
                        uiRectangle->box(FL_UP_BOX);
                        uiRectangle->down_box(FL_DOWN_BOX);
                        uiRectangle->color(FL_BACKGROUND_COLOR);
                        uiRectangle->selection_color(FL_YELLOW);
//image( image_Rectangle() );
                        uiRectangle->labeltype(FL_NORMAL_LABEL);
                        uiRectangle->labelfont(0);
                        uiRectangle->labelsize(14);
                        uiRectangle->labelcolor(FL_FOREGROUND_COLOR);
                        uiRectangle->align(Fl_Align(FL_ALIGN_CENTER));
                        uiRectangle->when(FL_WHEN_RELEASE);
                    } // Fl_Button* uiRectangle
                    { uiArrow = new Fl_Button(2, 263, 40, 35);
//tooltip(gettext("Arrow Tool"));
                        uiArrow->box(FL_UP_BOX);
                        uiArrow->down_box(FL_DOWN_BOX);
                        uiArrow->color(FL_BACKGROUND_COLOR);
                        uiArrow->selection_color(FL_YELLOW);
//image( image_Arrow() );
                        uiArrow->labeltype(FL_NORMAL_LABEL);
                        uiArrow->labelfont(0);
                        uiArrow->labelsize(14);
                        uiArrow->labelcolor(FL_FOREGROUND_COLOR);
//callback((Fl_Callback*)cb_uiArrow, (void*)(0));
                        uiArrow->align(Fl_Align(FL_ALIGN_CENTER));
                        uiArrow->when(FL_WHEN_RELEASE);
                    } // Fl_Button* uiArrow
                    { uiText = new Fl_Button(1, 298, 40, 35);
//tooltip(gettext("Text Tool.   Right click to edit previously stamped text."));
                        uiText->box(FL_UP_BOX);
                        uiText->down_box(FL_DOWN_BOX);
                        uiText->color(FL_BACKGROUND_COLOR);
                        uiText->selection_color(FL_YELLOW);
//image( image_Text() );
                        uiText->labeltype(FL_NORMAL_LABEL);
                        uiText->labelfont(8);
                        uiText->labelsize(24);
                        uiText->labelcolor(FL_BACKGROUND2_COLOR);
//callback((Fl_Callback*)cb_uiText, (void*)(0));
                        uiText->align(Fl_Align(FL_ALIGN_CENTER));
                        uiText->when(FL_WHEN_RELEASE);
                    } // Fl_Button* uiText
                    { uiPenColor = new Fl_Button(5, 337, 20, 20);
//tooltip(gettext("Pick Draw Color and Alpha"));
                        uiPenColor->box(FL_ENGRAVED_BOX);
                        uiPenColor->color((Fl_Color)3);
                        uiPenColor->selection_color((Fl_Color)3);
                        uiPenColor->labeltype(FL_NO_LABEL);
                        uiPenColor->labelfont(8);
                        uiPenColor->labelsize(24);
//callback((Fl_Callback*)cb_uiPenColor, (void*)(0));
                        uiPenColor->align(Fl_Align(FL_ALIGN_TEXT_OVER_IMAGE));
                    } // Fl_Button* uiPenColor
                    { uiOldPenColor = new Fl_Button(25, 357, 15, 16);
//tooltip(gettext("Old Color Used.  Click on the Arrow to alternate."));
                        uiOldPenColor->box(FL_ENGRAVED_BOX);
                        uiOldPenColor->color((Fl_Color)1);
                        uiOldPenColor->selection_color((Fl_Color)1);
                        uiOldPenColor->labeltype(FL_NO_LABEL);
                        uiOldPenColor->labelfont(8);
                        uiOldPenColor->labelsize(24);
                        uiOldPenColor->user_data((void*)(0));
                        uiOldPenColor->align(Fl_Align(FL_ALIGN_TEXT_OVER_IMAGE));
                    } // Fl_Button* uiOldPenColor
                    { uiFlipPenColor = new Fl_Button(25, 337, 16, 20);
//tooltip(gettext("Switch Colors"));
                        uiFlipPenColor->box(FL_NO_BOX);
                        uiFlipPenColor->color(FL_BACKGROUND_COLOR);
                        uiFlipPenColor->selection_color(FL_BACKGROUND_COLOR);
//image( image_Flip() );
                        uiFlipPenColor->labeltype(FL_NORMAL_LABEL);
                        uiFlipPenColor->labelfont(8);
                        uiFlipPenColor->labelsize(24);
                        uiFlipPenColor->labelcolor(FL_FOREGROUND_COLOR);
//callback((Fl_Callback*)cb_uiFlipPenColor, (void*)(0));
                        uiFlipPenColor->align(Fl_Align(FL_ALIGN_CENTER|FL_ALIGN_INSIDE));
                        uiFlipPenColor->when(FL_WHEN_RELEASE);
                    } // Fl_Button* uiFlipPenColor
                    { uiPenOpacity = new Fl_Slider(7, 373, 34, 19);
//tooltip(gettext("Opacity of pen stroke"));
                        uiPenOpacity->type(5);
                        uiPenOpacity->step(0.01);
                        uiPenOpacity->value(1);
//callback((Fl_Callback*)cb_uiPenOpacity);
                    } // Fl_Slider* uiPenOpacity
                    { Fl_Group* o = new Fl_Group(0, 396, 42, 71);
                        { uiUndoDraw = new Fl_Button(0, 396, 40, 35);
//tooltip(gettext("Undo Last Shape Drawn in Current Frame."));
                            uiUndoDraw->box(FL_UP_BOX);
                            uiUndoDraw->color(FL_BACKGROUND_COLOR);
                            uiUndoDraw->selection_color(FL_BACKGROUND_COLOR);
//image( image_Undo() );
                            uiUndoDraw->labeltype(FL_NORMAL_LABEL);
                            uiUndoDraw->labelfont(0);
                            uiUndoDraw->labelsize(14);
                            uiUndoDraw->labelcolor(FL_FOREGROUND_COLOR);
//callback((Fl_Callback*)cb_uiUndoDraw, (void*)(0));
                            uiUndoDraw->align(Fl_Align(FL_ALIGN_CENTER));
                            uiUndoDraw->when(FL_WHEN_RELEASE);
                            uiUndoDraw->deactivate();
                        } // Fl_Button* uiUndoDraw
                        { uiRedoDraw = new Fl_Button(0, 431, 40, 36);
//tooltip(gettext("Redo Previous Shape Drawn in Current Frame."));
                            uiRedoDraw->box(FL_UP_BOX);
                            uiRedoDraw->color(FL_BACKGROUND_COLOR);
                            uiRedoDraw->selection_color(FL_BACKGROUND_COLOR);
//image( image_Redo() );
                            uiRedoDraw->labeltype(FL_NORMAL_LABEL);
                            uiRedoDraw->labelfont(0);
                            uiRedoDraw->labelsize(14);
                            uiRedoDraw->labelcolor(FL_FOREGROUND_COLOR);
//callback((Fl_Callback*)cb_uiRedoDraw, (void*)(0));
                            uiRedoDraw->align(Fl_Align(FL_ALIGN_CENTER));
                            uiRedoDraw->when(FL_WHEN_RELEASE);
                            uiRedoDraw->deactivate();
                        } // Fl_Button* uiRedoDraw
                        o->end();
                    } // Fl_Group* o
                    uiToolsGroup->end();
                } // Fl_Group* uiToolsGroup
                { Fl_Viewport* o = uiView = new Fl_Viewport(42, 53, 378, 435);
                    uiView->box(FL_NO_BOX);
                    uiView->color(FL_BLACK);
                    uiView->selection_color(FL_BACKGROUND_COLOR);
                    uiView->labeltype(FL_NO_LABEL);
                    uiView->labelfont(0);
                    uiView->labelsize(14);
                    uiView->labelcolor(FL_FOREGROUND_COLOR);
                    uiView->align(Fl_Align(FL_ALIGN_TOP));
                    uiView->when(FL_WHEN_RELEASE);
                    uiView->end();
                    Fl_Group::current()->resizable(uiView);
                } // Fl_Viewport* uiView
                { Fl_Group* o = uiDockGroup = new Fl_Group(420, 53, 270, 435);
                    uiDockGroup->hide();
                    { uiResizableBar = new Fl_ResizableBar(420, 53, 10, 435);
                        uiResizableBar->box(FL_ENGRAVED_BOX);
                        uiResizableBar->color(FL_BACKGROUND_COLOR);
                        uiResizableBar->selection_color(FL_BACKGROUND_COLOR);
                        uiResizableBar->labeltype(FL_NORMAL_LABEL);
                        uiResizableBar->labelfont(0);
                        uiResizableBar->labelsize(14);
                        uiResizableBar->labelcolor(FL_FOREGROUND_COLOR);
                        uiResizableBar->align(Fl_Align(FL_ALIGN_CENTER));
                        uiResizableBar->when(FL_WHEN_RELEASE);
                    } // Fl_ResizableBar* uiResizableBar
                    { Fl_DockGroup* o = uiDock = new Fl_DockGroup(430, 53, 260, 435);
                        uiDock->box(FL_FLAT_BOX);
                        uiDock->color(FL_BACKGROUND_COLOR);
                        uiDock->selection_color(FL_BACKGROUND_COLOR);
                        uiDock->labeltype(FL_NORMAL_LABEL);
                        uiDock->labelfont(0);
                        uiDock->labelsize(14);
                        uiDock->labelcolor(FL_FOREGROUND_COLOR);
                        uiDock->align(Fl_Align(FL_ALIGN_TOP));
                        uiDock->when(FL_WHEN_RELEASE);
                        uiDock->end();
                        Fl_Group::current()->resizable(uiDock);
                    } // Fl_DockGroup* uiDock
                    uiViewGroup->fixed( o, 270 );
                    uiDockGroup->end();
                } // Fl_Group* uiDockGroup
                o->fixed( uiToolsGroup, uiToolsGroup->w() );
                uiViewGroup->end();
            } // Fl_Flex* uiViewGroup
            { uiTimelineGroup = new Fl_TimelineGroup(0, 488, 690, 30);
                uiTimelineGroup->box(FL_FLAT_BOX);
                uiTimelineGroup->color((Fl_Color)51);
                uiTimelineGroup->selection_color(FL_BACKGROUND_COLOR);
                uiTimelineGroup->labeltype(FL_NORMAL_LABEL);
                uiTimelineGroup->labelfont(0);
                uiTimelineGroup->labelsize(14);
                uiTimelineGroup->labelcolor(FL_FOREGROUND_COLOR);
                uiTimelineGroup->align(Fl_Align(FL_ALIGN_TOP));
                uiTimelineGroup->when(FL_WHEN_RELEASE);
                { uiTimeline = new Fl_TimelineWidget(0, 496, 690, 22);
                    uiTimeline->box(FL_NO_BOX);
                    uiTimeline->color(FL_BACKGROUND_COLOR);
                    uiTimeline->selection_color(FL_BACKGROUND_COLOR);
                    uiTimeline->labeltype(FL_NORMAL_LABEL);
                    uiTimeline->labelfont(0);
                    uiTimeline->labelsize(14);
                    uiTimeline->labelcolor(FL_FOREGROUND_COLOR);
                    uiTimeline->align(Fl_Align(FL_ALIGN_TOP));
                    uiTimeline->when(FL_WHEN_RELEASE);
                    uiTimeline->end();
                    Fl_Group::current()->resizable(uiTimeline);
                } // Fl_TimelineWidget* uiTimeline
                uiTimelineGroup->end();
            } // Fl_TimelineGroup* uiTimelineGroup
            ;
            uiTileGroup->end();
        } // Fl_Tile* uiTileGroup
        { uiBottomBar = new Fl_Group(0, 518, 690, 29);
            uiBottomBar->box(FL_FLAT_BOX);
            uiBottomBar->end();
        } // Fl_Group* uiBottomBar
        { uiPixelBar = new Fl_Group(0, 547, 690, 30);
            uiPixelBar->box(FL_FLAT_BOX);
            uiPixelBar->color(FL_BLACK);
            uiPixelBar->labelcolor((Fl_Color)17);
            uiPixelBar->end();
        } // Fl_Group* uiPixelBar
        { uiStatusGroup = new Fl_Group(0, 577, 690, 25);
            uiStatusGroup->color(FL_FOREGROUND_COLOR);
            uiStatusGroup->end();
        } // Fl_Group* uiStatusGroup
        o->fixed( uiMenuGroup, uiMenuGroup->h() );
        o->fixed( uiTopBar, uiTopBar->h() );
        o->fixed( uiBottomBar, uiBottomBar->h() );
        o->fixed( uiPixelBar, uiPixelBar->h() ); o->fixed( uiStatusGroup, uiStatusGroup->h() );
        uiRegion->end();
        Fl_Group::current()->resizable(uiRegion);
    } // Fl_Flex* uiRegion
    uiMain->size_range(690, 602);
    uiMain->end();
    uiMain->show();
    Fl::add_timeout(0.005, (Fl_Timeout_Handler)cb_Timeout, 0);
    return Fl::run();
}
